﻿<%@ WebHandler Language="C#" Class="ServerManager.Handlers.ClusterHandler" %>

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Net;
using System.Text;
using System.Web;
using System.Web.SessionState;
using Keysystems.WCF.Common;
using Newtonsoft.Json;

namespace ServerManager.Handlers
{
    public class ClusterHandler : IHttpHandler, IRequiresSessionState
    {
        public void ProcessRequest(HttpContext context)
        {
            string action = context.Request["ac"];
            object res = "NO RESPONSE";

            switch (action)
            {
                case "get":
                    res = UtilsConvert.ConvertJsonNS(GetProjects());
                    context.Response.ContentType = "text/plain";
                    break;

                case "getclusterapps":
                    res = UtilsConvert.ConvertJsonNS(GetClusterApps(context));
                    context.Response.ContentType = "text/plain";
                    break;

                case "updateApp":
                    res = UtilsConvert.ConvertJsonNS(UpdateApp(context));
                    context.Response.ContentType = "text/plain";
                    break;

                case "downloadApp":
                    res = UtilsConvert.ConvertJsonNS(DownLoad(context));
                    context.Response.ContentType = "text/plain";
                    break;

                case "resetClusterInfo":
                    res = UtilsConvert.ConvertJsonNS(ResetClusterInfo(context));
                    context.Response.ContentType = "text/plain";
                    break;
            }

            context.Response.Write(res);
            context.Response.End();
        }

        public bool IsReusable { get { return false; } }

        //--------------------------------------------------------------------------------------------------------
        /// <summary>
        /// Информация о сайтах/приложениях, возвращаемая соседним кластерам
        /// </summary>
        /// <returns></returns>
        private static object GetProjects()
        {
            try
            {
                return new AppSitesInfo().Categories;
            }
            catch (WarningException ex)
            {
                return ex;
            }
            catch (Exception ex)
            {
                throw ErrorManager.ErrorAJAX(ex);
            }
        }

        //--------------------------------------------------------------------------------------------------------
        /// <summary>
        /// Список таких же приложений на соседних серверах в кластере
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        private static object GetClusterApps(HttpContext context)
        {
            try
            {
                AppSite app = JsonConvert.DeserializeObject(context.Request["json"], typeof(AppSite)) as AppSite;
                if (app == null) return null;
                return GetAvailableClusterApp(app);
            }
            catch (Exception ex)
            {
                throw ErrorManager.ErrorAJAX(ex);
            }
        }

        /// <summary>
        /// Получить приложения с таким же названием с серверов в кластере
        /// </summary>
        private static Dictionary<string, AppSite> GetAvailableClusterApp(AppSite app)
        {
            var availableApps = new Dictionary<string, AppSite>
            {
                {"local", app}
            };

            var clusterConfig = Cluster.Current;
            if (clusterConfig.HasNeighbors &&
                clusterConfig.AppList.Contains(app.Path.Trim('/')))
            {
                foreach (var sitesInfo in clusterConfig.NeighborsSitesInfo)
                {
                    var serverUri = sitesInfo.Key;
                    var categories = sitesInfo.Value;
                    if (categories != null)
                    {
                        foreach (var category in categories)
                        {
                            if (!string.Equals(app.CategorySite, category.Site)) continue;

                            foreach (var appSite in category.AppSites)
                            {
                                if (!string.Equals(appSite.Path, app.Path)) continue;

                                availableApps.Add(serverUri, appSite);
                                break;
                            }
                        }
                    }

                    if (!availableApps.ContainsKey(serverUri))
                        availableApps.Add(serverUri, null);
                }
            }

            return availableApps;
        }

        //--------------------------------------------------------------------------------------------------------
        /// <summary>
        /// Обновить приложение в кластере
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        private static object UpdateApp(HttpContext context)
        {
            var errorMsg = string.Empty;

            string serverUri = JsonConvert.DeserializeObject(context.Request["serverUri"], typeof(string)) as string;
            AppSite appSite = JsonConvert.DeserializeObject(context.Request["appSite"], typeof(AppSite)) as AppSite;
            if (serverUri == null || appSite == null || string.Equals(appSite.CurrVersion, appSite.Version))
                return errorMsg;

            string result = string.Equals(serverUri, "local") ?
                DownLoadApp(appSite) : // Обновить локальное приложение
                UpdateClusterApp(serverUri, appSite); // Сделать запрос на соседний сервер для обновления приложения

            if (!result.IsNullOrEmpty())
                errorMsg += string.Format("{0}<br>", result);

            return errorMsg;
        }

        //--------------------------------------------------------------------------------------------------------
        /// <summary>
        /// Отправить на соседний в кластере сервер запрос на обновление выбранного приложения
        /// </summary>
        /// <returns></returns>
        private static string UpdateClusterApp(string serverUri, AppSite appSite)
        {
            try
            {
                var requestUri = new Uri(serverUri + "Handlers/ClusterHandler.ashx?ac=downloadApp");
                var request = (HttpWebRequest)WebRequest.Create(requestUri.AbsoluteUri);
                request.Proxy = new WebProxy();

                var postData = "json=" + Uri.EscapeDataString(UtilsConvert.ConvertJsonNS(appSite));
                var data = Encoding.ASCII.GetBytes(postData);

                request.Method = "POST";
                request.ContentType = "application/x-www-form-urlencoded";
                request.ContentLength = data.Length;

                using (var stream = request.GetRequestStream())
                {
                    stream.Write(data, 0, data.Length);
                }

                using (var response = request.GetResponse())
                using (var responseStream = response.GetResponseStream())
                {
                    if (responseStream != null)
                    {
                        using (var reader = new StreamReader(responseStream))
                        {
                            var responseText = reader.ReadToEnd();
                            string error = JsonConvert.DeserializeObject(responseText, typeof(string)) as string;
                            return error;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                return ex.Message;
            }

            return "";
        }

        //--------------------------------------------------------------------------------------------------------
        /// <summary>
        /// Обновить приложение до app.CurrVersion
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        private static string DownLoad(HttpContext context)
        {
            AppSite app = JsonConvert.DeserializeObject(context.Request["json"], typeof(AppSite)) as AppSite;
            return DownLoadApp(app);
        }

        /// <summary>
        /// Обновить приложение до app.CurrVersion
        /// </summary>
        /// <returns></returns>
        private static string DownLoadApp(AppSite app)
        {
            if (app != null)
            {
                app.DownLoadNewVersion();

                if (app.Mode != "edit" && app.Mode != "new" || app.Mode == null)
                {
                    app.AdjustByParams();
                    if (app.AppName != "Keysystems.BudgetNext")
                        app.SaveWebConfig();

                    var msg = app.Install();
                    return app.Errors + msg;
                }
            }
            return "";
        }

        //--------------------------------------------------------------------------------------------------------
        /// <summary>
        /// Сброс информации о кластере и обновление категории
        /// </summary>
        /// <returns></returns>
        private static object ResetClusterInfo(HttpContext context)
        {
            try
            {
                Cluster.Current.ResetClusterInfo();
                var idx = int.Parse(context.Request.Params["idx"]);
                return AppSitesInfo.getCatProp(idx);
            }
            catch (Exception ex)
            {
                throw ErrorManager.ErrorAJAX(ex);
            }
        }
    }
}